# Replacement for the normal add_library() command. The syntax remains the same
# in that the first argument is the target name, and the following arguments
# are the source files to use when building the target.
macro(RMDB_ADD_LIBRARY)
    set(options)
    set(oneValueArgs)
    set(multiValueArgs NAME SRCS PRIVATE_LINK_LIBS PUBLIC_LINK_LIBS)
	cmake_parse_arguments(RMDB_ADD_LIBRARY "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})
	add_library(${RMDB_ADD_LIBRARY_NAME} STATIC ${RMDB_ADD_LIBRARY_SRCS})
	# set_target_properties(${RMDB_ADD_LIBRARY_NAME} PROPERTIES FOLDER
    #     ${RMDB_TARGETS_ROOT_FOLDER}/${FOLDER_NAME})
    # if(CLANG_TIDY_EXE)
    #     set_target_properties(${RMDB_ADD_LIBRARY_NAME}
    #         PROPERTIES CXX_CLANG_TIDY "${CLANG_TIDY_EXE};-header-filter=.*")
    # endif()
    target_link_libraries(${RMDB_ADD_LIBRARY_NAME}
        PRIVATE ${RMDB_ADD_LIBRARY_PRIVATE_LINK_LIBS}
        PUBLIC ${RMDB_ADD_LIBRARY_PUBLIC_LINK_LIBS})
endmacro(RMDB_ADD_LIBRARY)

# Replacement for the normal add_executable() command. The syntax remains the
# same in that the first argument is the target name, and the following
# arguments are the source files to use when building the target.
macro(RMDB_ADD_EXECUTABLE)
    set(options)
    set(oneValueArgs)
    set(multiValueArgs NAME SRCS LINK_LIBS)
    cmake_parse_arguments(RMDB_ADD_EXECUTABLE "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})
    add_executable(${RMDB_ADD_EXECUTABLE_NAME} ${RMDB_ADD_EXECUTABLE_SRCS})
	# set_target_properties(${RMDB_ADD_EXECUTABLE_NAME} PROPERTIES FOLDER
    #     ${RMDB_TARGETS_ROOT_FOLDER}/${FOLDER_NAME})
    target_link_libraries(${RMDB_ADD_EXECUTABLE_NAME} ${RMDB_ADD_EXECUTABLE_LINK_LIBS})
	# if(VCPKG_BUILD)
    #     install(TARGETS ${RMDB_ADD_EXECUTABLE_NAME} DESTINATION tools/)
    # else()
    #     install(TARGETS ${RMDB_ADD_EXECUTABLE_NAME} DESTINATION bin/)
    # endif()
    # if(CLANG_TIDY_EXE)
    #     set_target_properties(${RMDB_ADD_EXECUTABLE_NAME}
    #         PROPERTIES CXX_CLANG_TIDY "${CLANG_TIDY_EXE};-header-filter=.*")
    # endif()
endmacro(RMDB_ADD_EXECUTABLE)

set(TEST_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/test)

# Wrapper for test executables.
macro(RMDB_ADD_TEST)
    set(options)
    set(oneValueArgs)
    set(multiValueArgs NAME SRCS LINK_LIBS)
    cmake_parse_arguments(RMDB_ADD_TEST "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})
    if(TESTS_ENABLED)
        # ${ARGN} will store the list of link libraries.
        set(RMDB_ADD_TEST_NAME "rmdb_${FOLDER_NAME}_${RMDB_ADD_TEST_NAME}")
        add_executable(${RMDB_ADD_TEST_NAME} ${RMDB_ADD_TEST_SRCS})
		set_target_properties(${RMDB_ADD_TEST_NAME} PROPERTIES RUNTIME_OUTPUT_DIRECTORY
			${TEST_OUTPUT_DIRECTORY}/${FOLDER_NAME})# ${RMDB_TARGETS_ROOT_FOLDER}/${FOLDER_NAME})
        # if(CLANG_TIDY_EXE)
        #     set_target_properties(${RMDB_ADD_TEST_NAME}
        #         PROPERTIES CXX_CLANG_TIDY "${CLANG_TIDY_EXE};-header-filter=.*")
        # endif()
        target_link_libraries(${RMDB_ADD_TEST_NAME}
            ${RMDB_ADD_TEST_LINK_LIBS}
            GTest::gtest
            GTest::gtest_main)
		add_test("${RMDB_ADD_TEST_NAME}" ${TEST_OUTPUT_DIRECTORY}/${FOLDER_NAME}/${RMDB_ADD_TEST_NAME})
		# if(IS_MSVC)
        #     install(TARGETS ${RMDB_ADD_TEST_NAME} DESTINATION bin/)
        # endif()
    endif()
endmacro(RMDB_ADD_TEST)
