#include <algorithm>
#include <cstdio>
#include <gtest/gtest.h>
#include "lru_replacer.h"
#include <random>
#include <unordered_set>


TEST(LRUReplacer, MaxSize1) {
	LRUReplacer l(3);
	int value = -1;
	EXPECT_EQ(0, l.Size());

    // Scenario: unpin 5 elements
	// Should be: [2, 3, 4]
	for (int i = 0; i < 5; i++) {
		l.unpin(i);
	}

	EXPECT_EQ(3, l.Size());
    l.victim(&value);
    EXPECT_EQ(2, value);
    l.victim(&value);
    EXPECT_EQ(3, value);
    l.victim(&value);
    EXPECT_EQ(4, value);


    // Scenario: unpin 100 elements
	// Should be: [7, 6, 5]
	for (int i = 104; i >= 5; i--) {
		l.unpin(i);
	}

	EXPECT_EQ(3, l.Size());
    l.victim(&value);
    EXPECT_EQ(7, value);
    l.victim(&value);
    EXPECT_EQ(6, value);
    l.victim(&value);
    EXPECT_EQ(5, value);
}

TEST(LRUReplacer, MaxSize2) {
	std::random_device seed;
	std::ranlux24 engine(seed());

	size_t siz = engine() % 1000, loop = engine() % 100000;
	LRUReplacer l(siz);

	std::unordered_set<unsigned int> vis;

	for (unsigned int i = 0; i < loop; i++) {
		int node = engine();
		if (vis.find(node) != vis.end()) {
			i--;
		}

		vis.insert(node);
		l.unpin(node);
	}

	EXPECT_EQ(l.Size(), std::min(siz, loop));
}

TEST(LRUReplacer, SampleTest1) {
    LRUReplacer lru_replacer(7);

    // Scenario: unpin six elements, i.e. add them to the replacer.
    lru_replacer.unpin(1);
    lru_replacer.unpin(2);
    lru_replacer.unpin(3);
    lru_replacer.unpin(4);
    lru_replacer.unpin(5);
    lru_replacer.unpin(6);
    lru_replacer.unpin(1);
    EXPECT_EQ(6, lru_replacer.Size());

    // Scenario: get three victims from the lru.
    int value;
    lru_replacer.victim(&value);
    EXPECT_EQ(1, value);
    lru_replacer.victim(&value);
    EXPECT_EQ(2, value);
    lru_replacer.victim(&value);
    EXPECT_EQ(3, value);

    // Scenario: pin elements in the replacer.
    // Note that 3 has already been victimized, so pinning 3 should have no effect.
    lru_replacer.pin(3);
    lru_replacer.pin(4);
    EXPECT_EQ(2, lru_replacer.Size());

    // Scenario: unpin 4. We expect that the reference bit of 4 will be set to 1.
    lru_replacer.unpin(4);

    // Scenario: continue looking for victims. We expect these victims.
    lru_replacer.victim(&value);
    EXPECT_EQ(5, value);
    lru_replacer.victim(&value);
    EXPECT_EQ(6, value);
    lru_replacer.victim(&value);
    EXPECT_EQ(4, value);
}

